@@ -501,6 +501,12 @@ signals:
 private:
     void setup();
 
+    /**
+     * This function is called to let the user know that JuK has automatically enabled
+     * manual column width adjust mode.
+     */
+    void notifyUserColumnWidthModeChanged();
+
     /**
      * Load the playlist from a file.  \a fileName should be the absolute path.
      * \a fileInfo should point to the same file as \a fileName.  This is a
@@ -641,14 +647,14 @@ private:
     bool m_allowDuplicates;
     bool m_polished;
     bool m_applySharedSettings;
-    bool m_mousePressed;
+    bool m_columnWidthModeChanged;
 
     QValueList<int> m_weightDirty;
     bool m_disableColumnWidthUpdates;
 
     mutable int m_time;
-    mutable QValueList<PlaylistItem::Pointer> m_addTime;
-    mutable QValueList<PlaylistItem::Pointer> m_subtractTime;
+    mutable PlaylistItemList m_addTime;
+    mutable PlaylistItemList m_subtractTime;
 
     /**
      * The average minimum widths of columns to be used in balancing calculations.
